from decimal import Decimal, ROUND_DOWN
from datetime import datetime, timedelta
import re

from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.safestring import mark_safe
from django.views.generic import ListView

from django_tables2 import Table, Column, LinkColumn
from django_tables2.utils import A
from django_tables2.views import SingleTableMixin

from .models import Company, Person, Ticker, Transaction


class NumberColumn(Column):
    @staticmethod
    def _format_integer(integer, sep=','):
        """Pretty-print an integer at thousands, millions, etc.

        >>> NumberColumn._format_integer(1234567)
        '1,234,567'
        >>> NumberColumn._format_integer(-123456)
        '-123,456'
        >>> NumberColumn._format_integer(1234567, sep=' ')
        '1 234 567'
        """
        sign = ''
        if integer < 0:
            sign = '-'
        string = '{:d}'.format(abs(integer))
        return sign + sep.join(re.findall('(.{1,3})', string[::-1]))[::-1]

    def render(self, value):
        classes = ['number']
        if value < 0:
            classes.append('negative')
        if value > 0:
            classes.append('positive')
        if isinstance(value, Decimal):
            integer = int(value.quantize(Decimal('1.'), rounding=ROUND_DOWN))
            decimal = int(abs(value - integer) * 100)
            print value
            value = '{}.{:02d}'.format(self._format_integer(integer), decimal)
            print value
        elif isinstance(value, int):
            value = self._format_integer(value)
        else:
            raise NotImplementedError(type(value))
        return mark_safe(u'<span class="{}">{}</span>'.format(
                u' '.join(classes), value))


class TransactionTable(Table):
    id = LinkColumn('transaction', args=[A('pk')])
    person = LinkColumn('person', args=[A('person.pk')])
    date = LinkColumn(
        'date', kwargs=dict((x, A('date.{}'.format(x)))
                            for x in ['year', 'month', 'day']))
    ticker = LinkColumn('ticker', args=[A('ticker.pk')])
    shares = NumberColumn()
    value = NumberColumn()

    class Meta:
        model = Transaction
        order_by = '-date'

    def render_source(self, value):
        return mark_safe(u'<a href="{}">source</a>'.format(value))


class TransactionTableView(SingleTableMixin, ListView):
    model = Transaction
    table_class = TransactionTable


def _detail(request, title, transactions):
    table = TransactionTable(transactions, order_by=request.GET.get('sort'))
    table.paginate(page=request.GET.get('page', 1))
    return render_to_response(
        'table.html', {'title': title, 'table': table},
        context_instance=RequestContext(request))

def company(request, pk):
    tickers = Ticker.objects.filter(company__id=pk)
    #tks = [ticker.pk for ticker in tickers]
    return _detail(request, Company.objects.get(pk=pk),
                   Transaction.objects.filter(ticker__in=tickers))

def person(request, pk):
    return _detail(request, Person.objects.get(pk=pk),
                   Transaction.objects.filter(person__id=pk))

def ticker(request, pk):
    ticker = Ticker.objects.get(pk=pk)
    company = ticker.company
    url = reverse('company', args=[company.id])
    title = mark_safe(u'{} (<a href="{}">{}</a>)'.format(
            ticker, url, company))
    return _detail(request, title, Transaction.objects.filter(ticker__id=pk))

def date(request, **kwargs):
    for k,v in kwargs.items():
        kwargs[k] = int(v)
    date = datetime(**kwargs)
    next_date = date.date() + timedelta(days=1)
    return _detail(request, date.date().isoformat(),
                   Transaction.objects.filter(date__range=(date,next_date)))
